#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <string.h>
#include <sys/utsname.h>
#include "dat.h"

static FILE *fp;
static int nproc = 1;
static int sleepsec = 1;
typedef unsigned long long uvlong;

static uvlong
Xatoull(char *s)
{
	return strtoull(s, NULL, 0);
}

static int
xapm(int first)
{
	static int fd = -1, fdb = -1;
	int i, last = -1, curr = -1;

	if(first){
		fd = open("/proc/acpi/battery/BAT0/info", O_RDONLY);
		fdb = open("/proc/acpi/battery/BAT0/state", O_RDONLY);
		return 0;
	}
	if(fd == -1 || fdb == -1)
		return 0;

	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 3)
			continue;
		if(strcmp(tok[0], "last") == 0 && strcmp(tok[1], "full") == 0)
			last = atoi(tok[3]);
	}
	readfile(fdb);
	for(i = 0; i < nline; i++) {
		tokens(i);
		if(ntok < 3)
			continue;
		if(strcmp(tok[0], "remaining") == 0 && strcmp(tok[1], "capacity:") == 0)
			curr = atoi(tok[2]);
	}
			
	if(curr != -1 && last != -1)
		fprintf(fp, "battery =%d 100\n", (int)(((float)curr/(float)last)*100.0));
	return 0;
}
	
static int
xloadavg(int first)
{
	static int fd = -1;

	if(first){
		fd = open("/proc/loadavg", O_RDONLY);
		return 0;
	}

	readfile(fd);
	tokens(0);
	if(ntok >= 1)
		fprintf(fp, "load #%d %d\n", (int)(atof(tok[0])*1000), nproc*1000);
	return 0;
}

static int
xmeminfo(int first)
{
	int i;
	uvlong tot, used;
	uvlong mtot, mfree;
	uvlong stot, sfree;
	static int fd = -1;

	if(first){
		fd = open("/proc/meminfo", O_RDONLY);
		return 0;
	}

	readfile(fd);
	mtot = 0;
	stot = 0;
	mfree = 0;
	sfree = 0;
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 3)
			continue;
		tot = Xatoull(tok[1]);
		used = Xatoull(tok[2]);
		if(strcmp(tok[0], "Mem:") == 0)
			fprintf(fp, "mem =%llu %llu\n", used/1024, tot/1024);
		else if(strcmp(tok[0], "Swap:") == 0)
			fprintf(fp, "swap =%llu %llu\n", used/1024, tot/1024);
		else if(strcmp(tok[0], "MemTotal:") == 0)
			mtot = Xatoull(tok[1]);					/* kb */
		else if(strcmp(tok[0], "MemFree:") == 0)
			mfree += Xatoull(tok[1]);
		else if(strcmp(tok[0], "Buffers:") == 0)
			mfree += Xatoull(tok[1]);
		else if(strcmp(tok[0], "Cached:") == 0){
			mfree += Xatoull(tok[1]);
			if(mtot < mfree)
				continue;
			fprintf(fp, "mem =%llu %llu\n", mtot-mfree, mtot);
		}else if(strcmp(tok[0], "SwapTotal:") == 0)
			stot = Xatoull(tok[1]);					/* kb */
		else if(strcmp(tok[0], "SwapFree:") == 0){
			sfree = Xatoull(tok[1]);
			if(stot < sfree)
				continue;
			fprintf(fp, "swap =%llu %llu\n", stot-sfree, stot);
		}
	}
	return 0;
}

static char *
getfile(char *fmt, ...)
{
	int fd, n;
	va_list ap;
	char path[128];
	static char buf[1024];

	va_start(ap, fmt);
	vsnprintf(path, sizeof(path), fmt, ap);
	va_end(ap);

	if((fd = open(path, O_RDONLY)) < 0)
		return "";
	if((n = read(fd, buf, sizeof(buf)-1)) < 0){
		close(fd);
		return "";
	}
	close(fd);

	buf[n] = 0;
	if(n > 0 && buf[n-1] == '\n')
		buf[n-1] = 0;
	return buf;
}


static int
xnet(int first)
{
	int i;
	uvlong totb, totp, tote, totin, totou, totinb, totoub, totmtu, totkbps;
	uvlong b, p, e, in, ou, inb, oub;
	char *q, *state, *mtu, *kbps;
	static int fd = -1;

	if(first){
		fd = open("/proc/net/dev", O_RDONLY);
		return 0;
	}

	readfile(fd);
	totb = 0;
	tote = 0;
	totp = 0;
	totin = 0;
	totou = 0;
	totinb = 0;
	totoub = 0;
	totmtu = 0;
	totkbps = 0;
	for(i=0; i<nline; i++){
		if((q = strchr(line[i], ':')) != NULL)
			*q = ' ';
		tokens(i);
		if(ntok < 8+8)
			continue;
		state = getfile("/sys/class/net/%s/operstate", tok[0]);
		if(strcmp(state, "up") != 0)
			continue;
		if(strncmp(tok[0], "eth", 3) != 0 && strncmp(tok[0], "wlan", 4) != 0)
			continue;
		inb = Xatoull(tok[1]);
		oub = Xatoull(tok[9]);
		in = Xatoull(tok[2]);
		ou = Xatoull(tok[10]);
		b = inb+oub;
		p = in+ou;
		e = Xatoull(tok[3])+Xatoull(tok[11]);
		kbps = getfile("/sys/class/net/%s/speed", tok[0]);
		totkbps += strtoll(kbps, NULL, 10) * 10;
		mtu = getfile("/sys/class/net/%s/mtu", tok[0]);
		totmtu += atol(mtu);
		totb += b;
		totp += p;
		tote += e;
		totin += in;
		totou += ou;
		totinb += inb;
		totoub += oub;
	}
	fprintf(fp, "ether %llu %d\n", totp, totkbps*2);
	fprintf(fp, "etherin %llu %d\n", totin, totkbps);
	fprintf(fp, "etherout %llu %d\n", totou, totkbps);

	fprintf(fp, "etherb %llu %d\n", totb, totkbps*totmtu*2);
	fprintf(fp, "etherinb %llu %d\n", totinb, totkbps*totmtu);
	fprintf(fp, "etheroutb %llu %d\n", totoub, totkbps*totmtu);

	fprintf(fp, "ethererr %llu %d\n", tote, totkbps/10);
	return 0;
}

static int
xstat(int first)
{
	static int fd = -1;
	int i;

	if(first){
		fd = open("/proc/stat", O_RDONLY);
		return 0;
	}

	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 2)
			continue;
		if(strcmp(tok[0], "cpu") == 0 && ntok >= 5){
			fprintf(fp, "user %llu %d\n", Xatoull(tok[1]), nproc*100);
			fprintf(fp, "sys %llu %d\n", Xatoull(tok[3]), nproc*100);
			fprintf(fp, "cpu %llu %d\n", Xatoull(tok[1])+Xatoull(tok[3]), nproc*100);
			fprintf(fp, "idle %llu %d\n", Xatoull(tok[4]), nproc*100);
		}
		if(strcmp(tok[0], "intr") == 0)
			fprintf(fp, "intr %llu %d\n", Xatoull(tok[1]), nproc*2000);
		if(strcmp(tok[0], "ctxt") == 0)
			fprintf(fp, "context %llu %d\n", Xatoull(tok[1]), nproc*400);
		if(strcmp(tok[0], "processes") == 0)
			fprintf(fp, "fork %llu %d\n", Xatoull(tok[1]), nproc*40);
	}
	return 0;
}

static int
xvmstat(int first)
{
	static int fd = -1;
	int i;

	if(first){
		fd = open("/proc/vmstat", O_RDONLY);
		return 0;
	}

	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 2)
			continue;
		if(strcmp(tok[0], "pgfault") == 0)
			fprintf(fp, "fault %llu 400000\n", Xatoull(tok[1]), nproc*2000);
	}
	return 0;
}

static int
xwireless(int first)
{
	static int fd = -1;
	int i;
	
	if(first){
		fd = open("/proc/net/wireless", O_RDONLY);
		return 0;
	}

	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 3)
			continue;
		if(strcmp(tok[0], "wlan0:") == 0)
			fprintf(fp, "802.11 =%llu 100\n", Xatoull(tok[2]));
	}
	return 0;
}

static int
xlinux(int first)
{
	FILE *pp;
	char *p, buf[16];
	struct utsname u;

	if(first){
		fp = fdopen(1, "w");
		setvbuf(fp, NULL, _IOFBF, 8192);

		if((pp = popen("nproc", "r")) != NULL){
			if(fgets(buf, sizeof(buf), pp))
				nproc = atoi(buf);
			pclose(pp);
		}
		uname(&u);

		if((p = strchr(u.release, '-')) != NULL)
			*p = 0;
		fprintf(fp, "info	'%s %s, %s' %d\n", u.sysname, u.release, u.machine, nproc);
		return 0;
	}

	fflush(fp);
	if(ferror(fp))
		return -1;
	sleep(sleepsec);
	return 0;
}

int (*statfn[])(int) =
{
	xlinux,
	xapm,
	xloadavg,
	xmeminfo,
	xnet,
	xstat,
	xvmstat,
	xwireless,
	0
};

