#include <u.h>
#include <libc.h>
#include <bio.h>
#include "dat.h"

static Biobuf bout;

static int
xmemory(int first)
{
	int i;
	char *p;
	vlong muse, mtot, suse, stot;
	static int fd = -1;

	if(first){
		fd = open("/dev/swap", OREAD);
		return 0;
	}

	readfile(fd);
	suse = 0;
	stot = 0;
	muse = 0;
	mtot = 0;
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 2)
			continue;
		if(strcmp(tok[1], "user") == 0){
			muse = strtoll(tok[0], &p, 10);	
			if(*p == '/')
				mtot = atoll(p+1);
		}
		else if(strcmp(tok[1], "swap") == 0){
			suse = strtoll(tok[0], &p, 10);	
			if(*p == '/')
				stot = atoll(p+1);
		}
	}

	if(muse < mtot)
		Bprint(&bout, "mem =%lld %lld\n", muse, mtot);
	if(suse < stot)
		Bprint(&bout, "swap =%lld %lld\n", suse, stot);
	return 0;
}


static int
xether(int first)
{
	char *s;
	int i, n, fd;
	vlong tote, totin, totou, totmbps;
	static long nent = 0;
	static Dir *dir = nil;

	if(first){
		fd = open("/net", OREAD);
		nent = dirreadall(fd, &dir);
		close(fd);
		return 0;
	}

	if(dir == nil || nent == 0)
		return 0;

	tote = 0;
	totin = 0;
	totou = 0;
	totmbps = 0;
	for(n = 0; n < nent; n++){
		if(strncmp(dir[n].name, "ether", 5) != 0)
			continue;
		if((s = smprint("/net/%s/stats", dir[n].name)) == nil)
			sysfatal("No memory\n");
		fd = open(s, OREAD);
		free(s);
		if(fd == -1)
			continue;

		readfile(fd);
		for(i=0; i<nline; i++){
			tokens(i);
			if(ntok < 2)
				continue;
			if(strcmp(tok[0], "in:") == 0)
				totin += atoll(tok[1]);
			if(strcmp(tok[0], "out:") == 0)
				totou += atoll(tok[1]);
			if(strcmp(tok[0], "crc errs:") == 0)
				tote += atoll(tok[1]);
			if(strcmp(tok[0], "overflows:") == 0)
				tote += atoll(tok[1]);
			if(strcmp(tok[0], "framing errs:") == 0)
				tote += atoll(tok[1]);
			if(strcmp(tok[0], "buffer errs:") == 0)
				tote += atoll(tok[1]);
			if(strcmp(tok[0], "output errs:") == 0)
				tote += atoll(tok[1]);
			if(strcmp(tok[0], "mbps:") == 0)
				totmbps += atoll(tok[1]) * 80;	/* 80 because 8 bits/byte */
		}
		close(fd);
	}

	Bprint(&bout, "ether %lld %d\n", totin+totou, totmbps*2);
	Bprint(&bout, "etherin %lld %d\n", totin, totmbps);
	Bprint(&bout, "etherout %lld %d\n", totou, totmbps);
	Bprint(&bout, "ethererr %lld %d\n", tote, 1000);
	return 0;
}


static int
xwireless(int first)
{
	static int fd = -1;
	int i;
	
	if(first){
		fd = open("/net/ether0/ifstats", OREAD);
		return 0;
	}

	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 2)
			continue;
		if(strcmp(tok[0], "Signal:") == 0)
			Bprint(&bout, "802.11 =%lld 100\n", atoll(tok[2]));
	}
	return 0;
}

static int
xfork(int first)
{
	char *p;
	Dir *d;
	uvlong n;
	static int fd = -1;
	static int nproc = 1;
	
	if(first){
		if((p = getenv("NPROC")) == nil){
			nproc = atoi(p);
			free(p);
		}
		fd = open("/proc", OREAD);
		return 0;
	}

	seek(fd, 0, 0);
	d = nil;
	n = dirreadall(fd, &d);
	free(d);

	Bprint(&bout, "fork =%lld %d\n", n, 5000 * nproc);
	return 0;
}

static int
xsysstats(int first)
{
	static int fd = -1;
	int i, n;
	vlong ctxsw, nintr, calls, pgflt, tlbflt, flbprg, loadav, idle, itime;

	if(first){
		fd = open("/dev/sysstat", ORDWR);
		return 0;
	}

	n = 0;
	ctxsw = 0;
	nintr = 0;
	calls = 0;
	pgflt = 0;
	tlbflt = 0;
	flbprg = 0;
	loadav = 0;
	idle = 0;
	itime = 0;
	readfile(fd);
	for(i=0; i<nline; i++){
		tokens(i);
		if(ntok < 2)
			continue;
		ctxsw += atoll(tok[1]);
		nintr += atoll(tok[2]);
		calls += atoll(tok[3]);
		pgflt += atoll(tok[4]);
		tlbflt += atoll(tok[5]);
		flbprg += atoll(tok[6]);
		loadav += atoll(tok[7]);
		idle += atoll(tok[8]);
		itime += atoll(tok[9]);
		n++;
	}

	Bprint(&bout, "context %lld %d\n", ctxsw, 15000 * n);
	Bprint(&bout, "fault %lld %d\n", pgflt, 5000 * n);
	Bprint(&bout, "syscall %lld %d\n", calls, 30000 * n);
	Bprint(&bout, "load =%lld %d\n", loadav, 2000 * n);
	Bprint(&bout, "idle =%lld 100\n", idle);
	Bprint(&bout, "intr  %lld 1500\n", nintr);
	return 0;
}

static int
xplan9(int first)
{
	int fd, n;
	char *p, buf[1024];

	n = 0;
	*buf = 0;
	if(first){
		if((fd = open("/dev/osversion", OREAD)) != -1){
			readfile(fd);
			tokens(0);
			if(ntok == 1)
				snprint(buf, sizeof(buf), "Plan9 %s", tok[0]);
			close(fd);
		}
		n = strlen(buf);

		if((fd = open("/dev/cputype", OREAD)) != -1){
			readfile(fd);
			tokens(0);
			if(ntok == 2)
				snprint(buf+n, sizeof(buf)-n, ", %s %.1fGhz", tok[0], atof(tok[1])/1000.0+0.05);
			close(fd);
		}

		Binit(&bout, 1, OWRITE);
		if((p = getenv("NPROC")) == 0)
			p = "1";
		Bprint(&bout, "info	'%s' %s\n", buf, p);
		return 0;
	}

	if(Bflush(&bout) < 0)
		return -1;
	sleep(900);
	return 0;
}

int (*statfn[])(int) =
{
	xplan9,
	xmemory,
	xfork,
	xsysstats,
	xether,
	xwireless,
	0
};

