#include <u.h>
#include <libc.h>
#include <thread.h>

typedef struct Execjob Execjob;
struct Execjob {
	int *fd;
	char *cmd;
	char **argv;
	Channel *c;
};

static void
execproc(void *v)
{
	Execjob *e;

	e = v;
	rfork(RFFDG);
	dup(e->fd[0], 0);
	dup(e->fd[1], 1);
	dup(e->fd[2], 2);
	procexec(e->c, e->cmd, e->argv);
	threadexits(nil);
}

int
threadspawn(int fd[3], char *cmd, char *argv[])
{
	int pid;
	Execjob e;

	e.fd = fd;
	e.cmd = cmd;
	e.argv = argv;
	e.c = chancreate(sizeof(ulong), 0);
	proccreate(execproc, &e, 65536);
	close(fd[0]);
	close(fd[1]);
	close(fd[2]);
	pid = recvul(e.c);
	chanfree(e.c);
	return pid;
}

int
threadspawnl(int fd[3], char *cmd, ...)
{
	char **argv, *s;
	int n, pid;
	va_list arg;

	va_start(arg, cmd);
	for(n=0; va_arg(arg, char*) != nil; n++)
		;
	n++;
	va_end(arg);

	argv = malloc(n*sizeof(argv[0]));
	if(argv == nil)
		return -1;

	va_start(arg, cmd);
	for(n=0; (s=va_arg(arg, char*)) != nil; n++)
		argv[n] = s;
	argv[n] = 0;
	va_end(arg);

	pid = threadspawn(fd, cmd, argv);
	free(argv);
	return pid;
}
